@@ -5,8 +5,6 @@ use crate::{
     types::{AsyncPushSender, RedisError, RedisResult, Value},
     AsyncConnectionConfig, Client, Cmd, ToRedisArgs,
 };
-#[cfg(all(not(feature = "tokio-comp"), feature = "async-std-comp"))]
-use ::async_std::net::ToSocketAddrs;
 use arc_swap::ArcSwap;
 use futures::{
     future::{self, Shared},
@@ -14,11 +12,13 @@ use futures::{
 };
 use futures_util::future::BoxFuture;
 use std::sync::Arc;
-use tokio_retry::strategy::{jitter, ExponentialBackoff};
-use tokio_retry::Retry;
+use tokio_retry2::{
+    strategy::{jitter, ExponentialBackoff},
+    MapErr, Retry,
+};
 
 /// ConnectionManager is the configuration for reconnect mechanism and request timing
-#[derive(Clone, Debug, Default)]
+#[derive(Clone, Debug)]
 pub struct ConnectionManagerConfig {
     /// The resulting duration is calculated by taking the base to the `n`-th power,
     /// where `n` denotes the number of past attempts.
@@ -32,9 +32,9 @@ pub struct ConnectionManagerConfig {
     /// Apply a maximum delay between connection attempts. The delay between attempts won't be longer than max_delay milliseconds.
     max_delay: Option<u64>,
     /// The new connection will time out operations after `response_timeout` has passed.
-    response_timeout: std::time::Duration,
+    response_timeout: Option<std::time::Duration>,
     /// Each connection attempt to the server will time out after `connection_timeout`.
-    connection_timeout: std::time::Duration,
+    connection_timeout: Option<std::time::Duration>,
     /// sender channel for push values
     push_sender: Option<AsyncPushSender>,
 }
@@ -43,20 +43,12 @@ impl ConnectionManagerConfig {
     const DEFAULT_CONNECTION_RETRY_EXPONENT_BASE: u64 = 2;
     const DEFAULT_CONNECTION_RETRY_FACTOR: u64 = 100;
     const DEFAULT_NUMBER_OF_CONNECTION_RETRIES: usize = 6;
-    const DEFAULT_RESPONSE_TIMEOUT: std::time::Duration = std::time::Duration::MAX;
-    const DEFAULT_CONNECTION_TIMEOUT: std::time::Duration = std::time::Duration::MAX;
+    const DEFAULT_RESPONSE_TIMEOUT: Option<std::time::Duration> = None;
+    const DEFAULT_CONNECTION_TIMEOUT: Option<std::time::Duration> = None;
 
     /// Creates a new instance of the options with nothing set
     pub fn new() -> Self {
-        Self {
-            exponent_base: Self::DEFAULT_CONNECTION_RETRY_EXPONENT_BASE,
-            factor: Self::DEFAULT_CONNECTION_RETRY_FACTOR,
-            number_of_retries: Self::DEFAULT_NUMBER_OF_CONNECTION_RETRIES,
-            max_delay: None,
-            response_timeout: Self::DEFAULT_RESPONSE_TIMEOUT,
-            connection_timeout: Self::DEFAULT_CONNECTION_TIMEOUT,
-            push_sender: None,
-        }
+        Self::default()
     }
 
     /// A multiplicative factor that will be applied to the retry delay.
@@ -91,7 +83,7 @@ impl ConnectionManagerConfig {
         mut self,
         duration: std::time::Duration,
     ) -> ConnectionManagerConfig {
-        self.response_timeout = duration;
+        self.response_timeout = Some(duration);
         self
     }
 
@@ -100,7 +92,7 @@ impl ConnectionManagerConfig {
         mut self,
         duration: std::time::Duration,
     ) -> ConnectionManagerConfig {
-        self.connection_timeout = duration;
+        self.connection_timeout = Some(duration);
         self
     }
 
@@ -110,6 +102,21 @@ impl ConnectionManagerConfig {
         self
     }
 }
+
+impl Default for ConnectionManagerConfig {
+    fn default() -> Self {
+        Self {
+            exponent_base: Self::DEFAULT_CONNECTION_RETRY_EXPONENT_BASE,
+            factor: Self::DEFAULT_CONNECTION_RETRY_FACTOR,
+            number_of_retries: Self::DEFAULT_NUMBER_OF_CONNECTION_RETRIES,
+            max_delay: None,
+            response_timeout: Self::DEFAULT_RESPONSE_TIMEOUT,
+            connection_timeout: Self::DEFAULT_CONNECTION_TIMEOUT,
+            push_sender: None,
+        }
+    }
+}
+
 /// A `ConnectionManager` is a proxy that wraps a [multiplexed
 /// connection][multiplexed-connection] and automatically reconnects to the
 /// server when necessary.
@@ -270,9 +277,13 @@ impl ConnectionManager {
             retry_strategy = retry_strategy.max_delay(std::time::Duration::from_millis(max_delay));
         }
 
-        let mut connection_config = AsyncConnectionConfig::new()
-            .set_connection_timeout(config.connection_timeout)
-            .set_response_timeout(config.response_timeout);
+        let mut connection_config = AsyncConnectionConfig::new();
+        if let Some(connection_timeout) = config.connection_timeout {
+            connection_config = connection_config.set_connection_timeout(connection_timeout);
+        }
+        if let Some(response_timeout) = config.response_timeout {
+            connection_config = connection_config.set_response_timeout(response_timeout);
+        }
 
         if let Some(push_sender) = config.push_sender.clone() {
             check_resp3!(
@@ -311,8 +322,11 @@ impl ConnectionManager {
     ) -> RedisResult<MultiplexedConnection> {
         let retry_strategy = exponential_backoff.map(jitter).take(number_of_retries);
         let connection_config = connection_config.clone();
-        Retry::spawn(retry_strategy, || {
-            client.get_multiplexed_async_connection_with_config(&connection_config)
+        Retry::spawn(retry_strategy, || async {
+            client
+                .get_multiplexed_async_connection_with_config(&connection_config)
+                .await
+                .map_transient_err()
         })
         .await
     }
