@@ -66,6 +66,94 @@ impl Client {
     pub fn get_connection_info(&self) -> &ConnectionInfo {
         &self.connection_info
     }
+
+    /// Constructs a new `Client` with parameters necessary to create a TLS connection.
+    ///
+    /// - `conn_info` - URL using the `rediss://` scheme.
+    /// - `tls_certs` - `TlsCertificates` structure containing:
+    ///     - `client_tls` - Optional `ClientTlsConfig` containing byte streams for
+    ///         - `client_cert` - client's byte stream containing client certificate in PEM format
+    ///         - `client_key` - client's byte stream containing private key in PEM format
+    ///     - `root_cert` - Optional byte stream yielding PEM formatted file for root certificates.
+    ///
+    /// If `ClientTlsConfig` ( cert+key pair ) is not provided, then client-side authentication is not enabled.
+    /// If `root_cert` is not provided, then system root certificates are used instead.
+    ///
+    /// # Examples
+    ///
+    /// ```no_run
+    /// use std::{fs::File, io::{BufReader, Read}};
+    ///
+    /// use redis::{Client, AsyncCommands as _, TlsCertificates, ClientTlsConfig};
+    ///
+    /// async fn do_redis_code(
+    ///     url: &str,
+    ///     root_cert_file: &str,
+    ///     cert_file: &str,
+    ///     key_file: &str
+    /// ) -> redis::RedisResult<()> {
+    ///     let root_cert_file = File::open(root_cert_file).expect("cannot open private cert file");
+    ///     let mut root_cert_vec = Vec::new();
+    ///     BufReader::new(root_cert_file)
+    ///         .read_to_end(&mut root_cert_vec)
+    ///         .expect("Unable to read ROOT cert file");
+    ///
+    ///     let cert_file = File::open(cert_file).expect("cannot open private cert file");
+    ///     let mut client_cert_vec = Vec::new();
+    ///     BufReader::new(cert_file)
+    ///         .read_to_end(&mut client_cert_vec)
+    ///         .expect("Unable to read client cert file");
+    ///
+    ///     let key_file = File::open(key_file).expect("cannot open private key file");
+    ///     let mut client_key_vec = Vec::new();
+    ///     BufReader::new(key_file)
+    ///         .read_to_end(&mut client_key_vec)
+    ///         .expect("Unable to read client key file");
+    ///
+    ///     let client = Client::build_with_tls(
+    ///         url,
+    ///         TlsCertificates {
+    ///             client_tls: Some(ClientTlsConfig{
+    ///                 client_cert: client_cert_vec,
+    ///                 client_key: client_key_vec,
+    ///             }),
+    ///             root_cert: Some(root_cert_vec),
+    ///         }
+    ///     )
+    ///     .expect("Unable to build client");
+    ///
+    ///     let connection_info = client.get_connection_info();
+    ///
+    ///     println!(">>> connection info: {connection_info:?}");
+    ///
+    ///     let mut con = client.get_multiplexed_async_connection().await?;
+    ///
+    ///     con.set("key1", b"foo").await?;
+    ///
+    ///     redis::cmd("SET")
+    ///         .arg(&["key2", "bar"])
+    ///         .exec_async(&mut con)
+    ///         .await?;
+    ///
+    ///     let result = redis::cmd("MGET")
+    ///         .arg(&["key1", "key2"])
+    ///         .query_async(&mut con)
+    ///         .await;
+    ///     assert_eq!(result, Ok(("foo".to_string(), b"bar".to_vec())));
+    ///     println!("Result from MGET: {result:?}");
+    ///
+    ///     Ok(())
+    /// }
+    /// ```
+    #[cfg(feature = "tls-rustls")]
+    pub fn build_with_tls<C: IntoConnectionInfo>(
+        conn_info: C,
+        tls_certs: TlsCertificates,
+    ) -> RedisResult<Client> {
+        let connection_info = conn_info.into_connection_info()?;
+
+        inner_build_with_tls(connection_info, tls_certs)
+    }
 }
 
 /// Options for creation of async connection
@@ -644,10 +732,11 @@ impl Client {
     where
         T: crate::aio::RedisRuntime,
     {
-        let (connection, driver) = self
+        let (mut connection, driver) = self
             .create_multiplexed_async_connection_inner::<T>(config)
             .await?;
-        T::spawn(driver);
+        let handle = T::spawn(driver);
+        connection.set_task_handle(handle);
         Ok(connection)
     }
 
@@ -686,102 +775,25 @@ impl Client {
         &self.connection_info
     }
 
-    /// Constructs a new `Client` with parameters necessary to create a TLS connection.
-    ///
-    /// - `conn_info` - URL using the `rediss://` scheme.
-    /// - `tls_certs` - `TlsCertificates` structure containing:
-    ///     - `client_tls` - Optional `ClientTlsConfig` containing byte streams for
-    ///         - `client_cert` - client's byte stream containing client certificate in PEM format
-    ///         - `client_key` - client's byte stream containing private key in PEM format
-    ///     - `root_cert` - Optional byte stream yielding PEM formatted file for root certificates.
-    ///
-    /// If `ClientTlsConfig` ( cert+key pair ) is not provided, then client-side authentication is not enabled.
-    /// If `root_cert` is not provided, then system root certificates are used instead.
-    ///
-    /// # Examples
-    ///
-    /// ```no_run
-    /// use std::{fs::File, io::{BufReader, Read}};
-    ///
-    /// use redis::{Client, AsyncCommands as _, TlsCertificates, ClientTlsConfig};
-    ///
-    /// async fn do_redis_code(
-    ///     url: &str,
-    ///     root_cert_file: &str,
-    ///     cert_file: &str,
-    ///     key_file: &str
-    /// ) -> redis::RedisResult<()> {
-    ///     let root_cert_file = File::open(root_cert_file).expect("cannot open private cert file");
-    ///     let mut root_cert_vec = Vec::new();
-    ///     BufReader::new(root_cert_file)
-    ///         .read_to_end(&mut root_cert_vec)
-    ///         .expect("Unable to read ROOT cert file");
-    ///
-    ///     let cert_file = File::open(cert_file).expect("cannot open private cert file");
-    ///     let mut client_cert_vec = Vec::new();
-    ///     BufReader::new(cert_file)
-    ///         .read_to_end(&mut client_cert_vec)
-    ///         .expect("Unable to read client cert file");
-    ///
-    ///     let key_file = File::open(key_file).expect("cannot open private key file");
-    ///     let mut client_key_vec = Vec::new();
-    ///     BufReader::new(key_file)
-    ///         .read_to_end(&mut client_key_vec)
-    ///         .expect("Unable to read client key file");
-    ///
-    ///     let client = Client::build_with_tls(
-    ///         url,
-    ///         TlsCertificates {
-    ///             client_tls: Some(ClientTlsConfig{
-    ///                 client_cert: client_cert_vec,
-    ///                 client_key: client_key_vec,
-    ///             }),
-    ///             root_cert: Some(root_cert_vec),
-    ///         }
-    ///     )
-    ///     .expect("Unable to build client");
-    ///
-    ///     let connection_info = client.get_connection_info();
-    ///
-    ///     println!(">>> connection info: {connection_info:?}");
-    ///
-    ///     let mut con = client.get_multiplexed_async_connection().await?;
-    ///
-    ///     con.set("key1", b"foo").await?;
-    ///
-    ///     redis::cmd("SET")
-    ///         .arg(&["key2", "bar"])
-    ///         .exec_async(&mut con)
-    ///         .await?;
-    ///
-    ///     let result = redis::cmd("MGET")
-    ///         .arg(&["key1", "key2"])
-    ///         .query_async(&mut con)
-    ///         .await;
-    ///     assert_eq!(result, Ok(("foo".to_string(), b"bar".to_vec())));
-    ///     println!("Result from MGET: {result:?}");
-    ///
-    ///     Ok(())
-    /// }
-    /// ```
-    #[cfg(feature = "tls-rustls")]
-    pub fn build_with_tls<C: IntoConnectionInfo>(
-        conn_info: C,
-        tls_certs: TlsCertificates,
-    ) -> RedisResult<Client> {
-        let connection_info = conn_info.into_connection_info()?;
-
-        inner_build_with_tls(connection_info, tls_certs)
-    }
-
     /// Returns an async receiver for pub-sub messages.
     #[cfg(any(feature = "tokio-comp", feature = "async-std-comp"))]
     // TODO - do we want to type-erase pubsub using a trait, to allow us to replace it with a different implementation later?
     pub async fn get_async_pubsub(&self) -> RedisResult<crate::aio::PubSub> {
-        #[allow(deprecated)]
-        self.get_async_connection()
-            .await
-            .map(|connection| connection.into_pubsub())
+        let connection = match Runtime::locate() {
+            #[cfg(feature = "tokio-comp")]
+            Runtime::Tokio => {
+                self.get_simple_async_connection::<crate::aio::tokio::Tokio>()
+                    .await?
+            }
+
+            #[cfg(feature = "async-std-comp")]
+            Runtime::AsyncStd => {
+                self.get_simple_async_connection::<crate::aio::async_std::AsyncStd>()
+                    .await?
+            }
+        };
+
+        crate::aio::PubSub::new(&self.connection_info.redis, connection).await
     }
 
     /// Returns an async receiver for monitor messages.
