@@ -1,7 +1,13 @@
 //! This module extends the library to support Redis Cluster.
 //!
-//! Note that this module does not currently provide pubsub
-//! functionality.
+//! The cluster connection is meant to abstract the fact that a cluster is composed of multiple nodes,
+//! and to provide an API which is as close as possible to that of a single node connection. In order to do that,
+//! the cluster connection maintains connections to each node in the Redis/ Valkey cluster, and can route
+//! requests automatically to the relevant nodes. In cases that the cluster connection receives indications
+//! that the cluster topology has changed, it will query nodes in order to find the current cluster topology.
+//! If it disconnects from some nodes, it will automatically reconnect to those nodes.
+//!
+//! Note that pubsub & push sending functionality is not currently provided by this module.
 //!
 //! # Example
 //! ```rust,no_run
@@ -232,7 +238,7 @@ where
             connections: RefCell::new(HashMap::new()),
             slots: RefCell::new(SlotMap::new(cluster_params.read_from_replicas)),
             auto_reconnect: RefCell::new(true),
-            read_timeout: RefCell::new(None),
+            read_timeout: RefCell::new(Some(cluster_params.response_timeout)),
             write_timeout: RefCell::new(None),
             initial_nodes: initial_nodes.to_vec(),
             cluster_params,
@@ -404,7 +410,7 @@ where
         let params = self.cluster_params.clone();
         let info = get_connection_info(node, params)?;
 
-        let mut conn = C::connect(info, None)?;
+        let mut conn = C::connect(info, Some(self.cluster_params.connection_timeout))?;
         if self.cluster_params.read_from_replicas {
             // If READONLY is sent to primary nodes, it will have no effect
             cmd("READONLY").exec(&mut conn)?;
@@ -989,7 +995,7 @@ pub(crate) fn get_connection_info(
         redis: RedisConnectionInfo {
             password: cluster_params.password,
             username: cluster_params.username,
-            protocol: cluster_params.protocol,
+            protocol: cluster_params.protocol.unwrap_or_default(),
             db: 0,
         },
     })
