@@ -1,10 +1,17 @@
-//! This module provides async functionality for Redis Cluster.
+//! This module provides async functionality for connecting to Redis / Valkey Clusters.
+//!
+//! The cluster connection is meant to abstract the fact that a cluster is composed of multiple nodes,
+//! and to provide an API which is as close as possible to that of a single node connection. In order to do that,
+//! the cluster connection maintains connections to each node in the Redis/ Valkey cluster, and can route
+//! requests automatically to the relevant nodes. In cases that the cluster connection receives indications
+//! that the cluster topology has changed, it will query nodes in order to find the current cluster topology.
+//! If it disconnects from some nodes, it will automatically reconnect to those nodes.
+//!
+//! Note that pubsub & push sending functionality is not currently provided by this module.
 //!
 //! By default, [`ClusterConnection`] makes use of [`MultiplexedConnection`] and maintains a pool
 //! of connections to each node in the cluster.
 //!
-//! Also note that pubsub functionality is not currently provided by this module.
-//!
 //! # Example
 //! ```rust,no_run
 //! use redis::cluster::ClusterClient;
@@ -72,7 +79,7 @@ use std::{
 mod request;
 mod routing;
 use crate::{
-    aio::{ConnectionLike, MultiplexedConnection},
+    aio::{ConnectionLike, MultiplexedConnection, Runtime, SharedHandleContainer},
     cluster::{get_connection_info, slot_cmd},
     cluster_client::ClusterParams,
     cluster_routing::{
@@ -80,12 +87,11 @@ use crate::{
         Slot, SlotMap,
     },
     cluster_topology::parse_slots,
+    types::closed_connection_error,
     Cmd, ConnectionInfo, ErrorKind, IntoConnectionInfo, RedisError, RedisFuture, RedisResult,
     Value,
 };
 
-#[cfg(all(not(feature = "tokio-comp"), feature = "async-std-comp"))]
-use crate::aio::{async_std::AsyncStd, RedisRuntime};
 use futures::{future::BoxFuture, prelude::*, ready};
 use log::{trace, warn};
 use rand::{seq::IteratorRandom, thread_rng};
@@ -97,7 +103,10 @@ use tokio::sync::{mpsc, oneshot, RwLock};
 /// underlying connections maintained for each node in the cluster, as well
 /// as common parameters for connecting to nodes and executing commands.
 #[derive(Clone)]
-pub struct ClusterConnection<C = MultiplexedConnection>(mpsc::Sender<Message<C>>);
+pub struct ClusterConnection<C = MultiplexedConnection> {
+    sender: mpsc::Sender<Message<C>>,
+    _task_handle: SharedHandleContainer,
+}
 
 impl<C> ClusterConnection<C>
 where
@@ -110,19 +119,19 @@ where
         ClusterConnInner::new(initial_nodes, cluster_params)
             .await
             .map(|inner| {
-                let (tx, mut rx) = mpsc::channel::<Message<_>>(100);
+                let (sender, mut receiver) = mpsc::channel::<Message<_>>(100);
                 let stream = async move {
-                    let _ = stream::poll_fn(move |cx| rx.poll_recv(cx))
+                    let _ = stream::poll_fn(move |cx| receiver.poll_recv(cx))
                         .map(Ok)
                         .forward(inner)
                         .await;
                 };
-                #[cfg(feature = "tokio-comp")]
-                tokio::spawn(stream);
-                #[cfg(all(not(feature = "tokio-comp"), feature = "async-std-comp"))]
-                AsyncStd::spawn(stream);
+                let _task_handle = SharedHandleContainer::new(Runtime::locate().spawn(stream));
 
-                ClusterConnection(tx)
+                ClusterConnection {
+                    sender,
+                    _task_handle,
+                }
             })
     }
 
@@ -130,7 +139,7 @@ where
     pub async fn route_command(&mut self, cmd: &Cmd, routing: RoutingInfo) -> RedisResult<Value> {
         trace!("send_packed_command");
         let (sender, receiver) = oneshot::channel();
-        self.0
+        self.sender
             .send(Message {
                 cmd: CmdArg::Cmd {
                     cmd: Arc::new(cmd.clone()), // TODO Remove this clone?
@@ -168,7 +177,7 @@ where
         route: SingleNodeRoutingInfo,
     ) -> RedisResult<Vec<Value>> {
         let (sender, receiver) = oneshot::channel();
-        self.0
+        self.sender
             .send(Message {
                 cmd: CmdArg::Pipeline {
                     pipeline: Arc::new(pipeline.clone()), // TODO Remove this clone?
@@ -179,11 +188,11 @@ where
                 sender,
             })
             .await
-            .map_err(|_| RedisError::from(io::Error::from(io::ErrorKind::BrokenPipe)))?;
+            .map_err(|_| closed_connection_error())?;
 
         receiver
             .await
-            .unwrap_or_else(|_| Err(RedisError::from(io::Error::from(io::ErrorKind::BrokenPipe))))
+            .unwrap_or_else(|_| Err(closed_connection_error()))
             .map(|response| match response {
                 Response::Multiple(values) => values,
                 Response::Single(_) => unreachable!(),
@@ -194,6 +203,9 @@ where
 type ConnectionFuture<C> = future::Shared<BoxFuture<'static, C>>;
 type ConnectionMap<C> = HashMap<String, ConnectionFuture<C>>;
 
+/// This is the internal representation of an async Redis Cluster connection. It stores the
+/// underlying connections maintained for each node in the cluster, as well
+/// as common parameters for connecting to nodes and executing commands.
 struct InnerCore<C> {
     conn_lock: RwLock<(ConnectionMap<C>, SlotMap)>,
     cluster_params: ClusterParams,
@@ -203,6 +215,9 @@ struct InnerCore<C> {
 
 type Core<C> = Arc<InnerCore<C>>;
 
+/// This is the sink for requests sent by the user.
+/// It holds the stream of requests which are "in flight", E.G. on their way to the server,
+/// and the inner representation of the connection.
 struct ClusterConnInner<C> {
     inner: Core<C>,
     state: ConnectionState,
