@@ -1044,6 +1044,11 @@ implement_commands! {
         cmd("OBJECT").arg("REFCOUNT").arg(key)
     }
 
+    /// Returns the name of the current connection as set by CLIENT SETNAME.
+    fn client_getname<>() {
+        cmd("CLIENT").arg("GETNAME")
+    }
+
     // ACL commands
 
     /// When Redis is configured to use an ACL file (with the aclfile
@@ -1109,6 +1114,13 @@ implement_commands! {
         cmd("ACL").arg("DELUSER").arg(usernames)
     }
 
+    /// Simulate the execution of a given command by a given user.
+    #[cfg(feature = "acl")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "acl")))]
+    fn acl_dryrun<K: ToRedisArgs, C: ToRedisArgs, A: ToRedisArgs>(username: K, command: C, args: A) {
+        cmd("ACL").arg("DRYRUN").arg(username).arg(command).arg(args)
+    }
+
     /// Shows the available ACL categories.
     #[cfg(feature = "acl")]
     #[cfg_attr(docsrs, doc(cfg(feature = "acl")))]
@@ -1420,6 +1432,42 @@ implement_commands! {
         cmd("XADD").arg(key).arg(id).arg(map)
     }
 
+
+    /// Add a stream message with options.
+    ///
+    /// Items can be any list type, e.g.
+    /// ```rust
+    /// // static items
+    /// let items = &[("key", "val"), ("key2", "val2")];
+    /// # use std::collections::BTreeMap;
+    /// // A map (Can be BTreeMap, HashMap, etc)
+    /// let mut map: BTreeMap<&str, &str> = BTreeMap::new();
+    /// map.insert("ab", "cd");
+    /// map.insert("ef", "gh");
+    /// map.insert("ij", "kl");
+    /// ```
+    ///
+    /// ```text
+    /// XADD key [NOMKSTREAM] [<MAXLEN|MINID> [~|=] threshold [LIMIT count]] <* | ID> field value [field value] ...
+    /// ```
+    #[cfg(feature = "streams")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "streams")))]
+    fn xadd_options<
+        K: ToRedisArgs, ID: ToRedisArgs, I: ToRedisArgs
+    >(
+        key: K,
+        id: ID,
+        items: I,
+        options: &'a streams::StreamAddOptions
+    ) {
+        cmd("XADD")
+            .arg(key)
+            .arg(options)
+            .arg(id)
+            .arg(items)
+    }
+
+
     /// Add a stream message while capping the stream at a maxlength.
     ///
     /// ```text
@@ -1563,7 +1611,7 @@ implement_commands! {
     /// ```text
     /// XCLAIM <key> <group> <consumer> <min-idle-time> <ID-1> <ID-2>
     ///     [IDLE <milliseconds>] [TIME <mstime>] [RETRYCOUNT <count>]
-    ///     [FORCE] [JUSTID]
+    ///     [FORCE] [JUSTID] [LASTID <lastid>]
     /// ```
     #[cfg(feature = "streams")]
     #[cfg_attr(docsrs, doc(cfg(feature = "streams")))]
@@ -2085,6 +2133,20 @@ implement_commands! {
         cmd("XTRIM").arg(key).arg(maxlen)
     }
 
+     /// Trim a stream `key` with full options
+     ///
+     /// ```text
+     /// XTRIM <key> <MAXLEN|MINID> [~|=] <threshold> [LIMIT <count>]  (Same as XADD MAXID|MINID options)
+     /// ```
+    #[cfg(feature = "streams")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "streams")))]
+    fn xtrim_options<K: ToRedisArgs>(
+        key: K,
+        options: &'a streams::StreamTrimOptions
+    ) {
+        cmd("XTRIM").arg(key).arg(options)
+    }
+
     // script commands
 
     /// Adds a prepared script command to the pipeline.
@@ -2246,6 +2308,7 @@ impl PubSubCommands for Connection {
 pub struct ScanOptions {
     pattern: Option<String>,
     count: Option<usize>,
+    scan_type: Option<String>,
 }
 
 impl ScanOptions {
@@ -2260,6 +2323,12 @@ impl ScanOptions {
         self.pattern = Some(p.into());
         self
     }
+
+    /// Limit the results to those with the given Redis type
+    pub fn with_type(mut self, t: impl Into<String>) -> Self {
+        self.scan_type = Some(t.into());
+        self
+    }
 }
 
 impl ToRedisArgs for ScanOptions {
@@ -2276,6 +2345,11 @@ impl ToRedisArgs for ScanOptions {
             out.write_arg(b"COUNT");
             out.write_arg_fmt(n);
         }
+
+        if let Some(t) = &self.scan_type {
+            out.write_arg(b"TYPE");
+            out.write_arg_fmt(t);
+        }
     }
 
     fn num_of_args(&self) -> usize {
@@ -2286,6 +2360,9 @@ impl ToRedisArgs for ScanOptions {
         if self.count.is_some() {
             len += 2;
         }
+        if self.scan_type.is_some() {
+            len += 2;
+        }
         len
     }
 }
@@ -2487,15 +2564,13 @@ impl ToRedisArgs for SetOptions {
 pub fn resp3_hello(connection_info: &RedisConnectionInfo) -> Cmd {
     let mut hello_cmd = cmd("HELLO");
     hello_cmd.arg("3");
-    if connection_info.password.is_some() {
+    if let Some(password) = &connection_info.password {
         let username: &str = match connection_info.username.as_ref() {
             None => "default",
             Some(username) => username,
         };
-        hello_cmd
-            .arg("AUTH")
-            .arg(username)
-            .arg(connection_info.password.as_ref().unwrap());
+        hello_cmd.arg("AUTH").arg(username).arg(password);
     }
+
     hello_cmd
 }
