@@ -104,7 +104,7 @@ pub fn parse_redis_url(input: &str) -> Option<url::Url> {
 
 /// TlsMode indicates use or do not use verification of certification.
 /// Check [ConnectionAddr](ConnectionAddr::TcpTls::insecure) for more.
-#[derive(Clone, Copy)]
+#[derive(Clone, Copy, PartialEq)]
 pub enum TlsMode {
     /// Secure verify certification.
     Secure,
@@ -189,6 +189,20 @@ impl ConnectionAddr {
             ConnectionAddr::Unix(_) => cfg!(unix),
         }
     }
+
+    #[cfg(feature = "cluster")]
+    pub(crate) fn tls_mode(&self) -> Option<TlsMode> {
+        match self {
+            ConnectionAddr::TcpTls { insecure, .. } => {
+                if *insecure {
+                    Some(TlsMode::Insecure)
+                } else {
+                    Some(TlsMode::Secure)
+                }
+            }
+            _ => None,
+        }
+    }
 }
 
 impl fmt::Display for ConnectionAddr {
@@ -545,6 +559,10 @@ pub struct Connection {
 
     /// This is used to manage Push messages in RESP3 mode.
     push_sender: Option<SyncPushSender>,
+
+    /// The number of messages that are expected to be returned from the server,
+    /// but the user no longer waits for - answers for requests that already returned a transient error.
+    messages_to_skip: usize,
 }
 
 /// Represents a pubsub connection.
@@ -920,71 +938,207 @@ pub(crate) fn create_rustls_config(
     }
 }
 
-fn connect_auth(
-    con: &mut Connection,
+fn authenticate_cmd(
     connection_info: &RedisConnectionInfo,
+    check_username: bool,
     password: &str,
-) -> RedisResult<()> {
+) -> Cmd {
     let mut command = cmd("AUTH");
-    if let Some(username) = &connection_info.username {
-        command.arg(username);
-    }
-    let err = match command.arg(password).query::<Value>(con) {
-        Ok(Value::Okay) => return Ok(()),
-        Ok(_) => {
-            fail!((
-                ErrorKind::ResponseError,
-                "Redis server refused to authenticate, returns Ok() != Value::Okay"
-            ));
+    if check_username {
+        if let Some(username) = &connection_info.username {
+            command.arg(username);
         }
-        Err(e) => e,
-    };
-    let err_msg = err.detail().ok_or((
-        ErrorKind::AuthenticationFailed,
-        "Password authentication failed",
-    ))?;
-    if !err_msg.contains("wrong number of arguments for 'auth' command") {
-        fail!((
-            ErrorKind::AuthenticationFailed,
-            "Password authentication failed",
-        ));
-    }
-
-    // fallback to AUTH version <= 5
-    let mut command = cmd("AUTH");
-    match command.arg(password).query::<Value>(con) {
-        Ok(Value::Okay) => Ok(()),
-        _ => fail!((
-            ErrorKind::AuthenticationFailed,
-            "Password authentication failed",
-        )),
     }
+    command.arg(password);
+    command
 }
 
 pub fn connect(
     connection_info: &ConnectionInfo,
     timeout: Option<Duration>,
 ) -> RedisResult<Connection> {
-    let con = ActualConnection::new(&connection_info.addr, timeout)?;
+    let con: ActualConnection = ActualConnection::new(&connection_info.addr, timeout)?;
     setup_connection(con, &connection_info.redis)
 }
 
-#[cfg(not(feature = "disable-client-setinfo"))]
-pub(crate) fn client_set_info_pipeline() -> Pipeline {
-    let mut pipeline = crate::pipe();
+pub(crate) struct ConnectionSetupComponents {
+    resp3_auth_cmd_idx: Option<usize>,
+    resp2_auth_cmd_idx: Option<usize>,
+    select_cmd_idx: Option<usize>,
+}
+
+pub(crate) fn connection_setup_pipeline(
+    connection_info: &RedisConnectionInfo,
+    check_username: bool,
+) -> (crate::Pipeline, ConnectionSetupComponents) {
+    let mut last_cmd_index = 0;
+
+    let mut get_next_command_index = |condition| {
+        if condition {
+            last_cmd_index += 1;
+            Some(last_cmd_index - 1)
+        } else {
+            None
+        }
+    };
+
+    let authenticate_with_resp3_cmd_index =
+        get_next_command_index(connection_info.protocol != ProtocolVersion::RESP2);
+    let authenticate_with_resp2_cmd_index = get_next_command_index(
+        authenticate_with_resp3_cmd_index.is_none() && connection_info.password.is_some(),
+    );
+    let select_db_cmd_index = get_next_command_index(connection_info.db != 0);
+
+    let mut pipeline = pipe();
+
+    if authenticate_with_resp3_cmd_index.is_some() {
+        pipeline.add_command(resp3_hello(connection_info));
+    } else if authenticate_with_resp2_cmd_index.is_some() {
+        pipeline.add_command(authenticate_cmd(
+            connection_info,
+            check_username,
+            connection_info.password.as_ref().unwrap(),
+        ));
+    }
+
+    if select_db_cmd_index.is_some() {
+        pipeline.cmd("SELECT").arg(connection_info.db);
+    }
+
+    // result is ignored, as per the command's instructions.
+    // https://redis.io/commands/client-setinfo/
+    #[cfg(not(feature = "disable-client-setinfo"))]
     pipeline
         .cmd("CLIENT")
         .arg("SETINFO")
         .arg("LIB-NAME")
         .arg("redis-rs")
         .ignore();
+    #[cfg(not(feature = "disable-client-setinfo"))]
     pipeline
         .cmd("CLIENT")
         .arg("SETINFO")
         .arg("LIB-VER")
         .arg(env!("CARGO_PKG_VERSION"))
         .ignore();
-    pipeline
+
+    (
+        pipeline,
+        ConnectionSetupComponents {
+            resp3_auth_cmd_idx: authenticate_with_resp3_cmd_index,
+            resp2_auth_cmd_idx: authenticate_with_resp2_cmd_index,
+            select_cmd_idx: select_db_cmd_index,
+        },
+    )
+}
+
+fn check_resp3_auth(result: &Value) -> RedisResult<()> {
+    if let Value::ServerError(err) = result {
+        return Err(get_resp3_hello_command_error(err.clone().into()));
+    }
+    Ok(())
+}
+
+#[derive(PartialEq)]
+pub(crate) enum AuthResult {
+    Succeeded,
+    ShouldRetryWithoutUsername,
+}
+
+fn check_resp2_auth(result: &Value) -> RedisResult<AuthResult> {
+    let err = match result {
+        Value::Okay => {
+            return Ok(AuthResult::Succeeded);
+        }
+        Value::ServerError(err) => err,
+        _ => {
+            return Err((
+                ErrorKind::ResponseError,
+                "Redis server refused to authenticate, returns Ok() != Value::Okay",
+            )
+                .into());
+        }
+    };
+
+    let err_msg = err.details().ok_or((
+        ErrorKind::AuthenticationFailed,
+        "Password authentication failed",
+    ))?;
+    if !err_msg.contains("wrong number of arguments for 'auth' command") {
+        return Err((
+            ErrorKind::AuthenticationFailed,
+            "Password authentication failed",
+        )
+            .into());
+    }
+    Ok(AuthResult::ShouldRetryWithoutUsername)
+}
+
+fn check_db_select(value: &Value) -> RedisResult<()> {
+    let Value::ServerError(err) = value else {
+        return Ok(());
+    };
+
+    match err.details() {
+        Some(err_msg) => Err((
+            ErrorKind::ResponseError,
+            "Redis server refused to switch database",
+            err_msg.to_string(),
+        )
+            .into()),
+        None => Err((
+            ErrorKind::ResponseError,
+            "Redis server refused to switch database",
+        )
+            .into()),
+    }
+}
+
+pub(crate) fn check_connection_setup(
+    results: Vec<Value>,
+    ConnectionSetupComponents {
+        resp3_auth_cmd_idx,
+        resp2_auth_cmd_idx,
+        select_cmd_idx,
+    }: ConnectionSetupComponents,
+) -> RedisResult<AuthResult> {
+    // can't have both values set
+    assert!(!(resp2_auth_cmd_idx.is_some() && resp3_auth_cmd_idx.is_some()));
+
+    if let Some(index) = resp3_auth_cmd_idx {
+        let Some(value) = results.get(index) else {
+            return Err((ErrorKind::ClientError, "Missing RESP3 auth response").into());
+        };
+        check_resp3_auth(value)?;
+    } else if let Some(index) = resp2_auth_cmd_idx {
+        let Some(value) = results.get(index) else {
+            return Err((ErrorKind::ClientError, "Missing RESP2 auth response").into());
+        };
+        if check_resp2_auth(value)? == AuthResult::ShouldRetryWithoutUsername {
+            return Ok(AuthResult::ShouldRetryWithoutUsername);
+        }
+    }
+
+    if let Some(index) = select_cmd_idx {
+        let Some(value) = results.get(index) else {
+            return Err((ErrorKind::ClientError, "Missing SELECT DB response").into());
+        };
+        check_db_select(value)?;
+    }
+
+    Ok(AuthResult::Succeeded)
+}
+
+fn execute_connection_pipeline(
+    rv: &mut Connection,
+    (pipeline, instructions): (crate::Pipeline, ConnectionSetupComponents),
+) -> RedisResult<AuthResult> {
+    if pipeline.len() == 0 {
+        return Ok(AuthResult::Succeeded);
+    }
+    let results = rv.req_packed_commands(&pipeline.get_packed_pipeline(), 0, pipeline.len())?;
+
+    check_connection_setup(results, instructions)
 }
 
 fn setup_connection(
@@ -998,34 +1152,14 @@ fn setup_connection(
         pubsub: false,
         protocol: connection_info.protocol,
         push_sender: None,
+        messages_to_skip: 0,
     };
 
-    if connection_info.protocol != ProtocolVersion::RESP2 {
-        let hello_cmd = resp3_hello(connection_info);
-        let val: RedisResult<Value> = hello_cmd.query(&mut rv);
-        if let Err(err) = val {
-            return Err(get_resp3_hello_command_error(err));
-        }
-    } else if let Some(password) = &connection_info.password {
-        connect_auth(&mut rv, connection_info, password)?;
+    if execute_connection_pipeline(&mut rv, connection_setup_pipeline(connection_info, true))?
+        == AuthResult::ShouldRetryWithoutUsername
+    {
+        execute_connection_pipeline(&mut rv, connection_setup_pipeline(connection_info, false))?;
     }
-    if connection_info.db != 0 {
-        match cmd("SELECT")
-            .arg(connection_info.db)
-            .query::<Value>(&mut rv)
-        {
-            Ok(Value::Okay) => {}
-            _ => fail!((
-                ErrorKind::ResponseError,
-                "Redis server refused to switch database"
-            )),
-        }
-    }
-
-    // result is ignored, as per the command's instructions.
-    // https://redis.io/commands/client-setinfo/
-    #[cfg(not(feature = "disable-client-setinfo"))]
-    let _: RedisResult<()> = client_set_info_pipeline().query(&mut rv);
 
     Ok(rv)
 }
@@ -1083,7 +1217,7 @@ pub trait ConnectionLike {
 
     /// Returns the connection status.
     ///
-    /// The connection is open until any `read_response` call received an
+    /// The connection is open until any `read` call received an
     /// invalid response from the server (most likely a closed or dropped
     /// connection, otherwise a Redis protocol error). When using unix
     /// sockets the connection is open until writing a command failed with a
@@ -1111,7 +1245,7 @@ impl Connection {
     /// Fetches a single response from the connection.  This is useful
     /// if used in combination with `send_packed_command`.
     pub fn recv_response(&mut self) -> RedisResult<Value> {
-        self.read_response()
+        self.read(true)
     }
 
     /// Sets the write timeout for the connection.
@@ -1235,68 +1369,80 @@ impl Connection {
         })
     }
 
-    /// Fetches a single response from the connection.
-    fn read_response(&mut self) -> RedisResult<Value> {
-        let result = match self.con {
-            ActualConnection::Tcp(TcpConnection { ref mut reader, .. }) => {
-                let result = self.parser.parse_value(reader);
-                self.try_send(&result);
-                result
+    fn close_connection(&mut self) {
+        // Notify the PushManager that the connection was lost
+        self.send_disconnect();
+        match self.con {
+            ActualConnection::Tcp(ref mut connection) => {
+                let _ = connection.reader.shutdown(net::Shutdown::Both);
+                connection.open = false;
             }
             #[cfg(all(feature = "tls-native-tls", not(feature = "tls-rustls")))]
-            ActualConnection::TcpNativeTls(ref mut boxed_tls_connection) => {
-                let reader = &mut boxed_tls_connection.reader;
-                let result = self.parser.parse_value(reader);
-                self.try_send(&result);
-                result
+            ActualConnection::TcpNativeTls(ref mut connection) => {
+                let _ = connection.reader.shutdown();
+                connection.open = false;
             }
             #[cfg(feature = "tls-rustls")]
-            ActualConnection::TcpRustls(ref mut boxed_tls_connection) => {
-                let reader = &mut boxed_tls_connection.reader;
-                let result = self.parser.parse_value(reader);
-                self.try_send(&result);
-                result
+            ActualConnection::TcpRustls(ref mut connection) => {
+                let _ = connection.reader.get_mut().shutdown(net::Shutdown::Both);
+                connection.open = false;
             }
             #[cfg(unix)]
-            ActualConnection::Unix(UnixConnection { ref mut sock, .. }) => {
-                let result = self.parser.parse_value(sock);
-                self.try_send(&result);
-                result
-            }
-        };
-        // shutdown connection on protocol error
-        if let Err(e) = &result {
-            let shutdown = match e.as_io_error() {
-                Some(e) => e.kind() == io::ErrorKind::UnexpectedEof,
-                None => false,
-            };
-            if shutdown {
-                // Notify the PushManager that the connection was lost
-                self.send_disconnect();
-                match self.con {
-                    ActualConnection::Tcp(ref mut connection) => {
-                        let _ = connection.reader.shutdown(net::Shutdown::Both);
-                        connection.open = false;
-                    }
-                    #[cfg(all(feature = "tls-native-tls", not(feature = "tls-rustls")))]
-                    ActualConnection::TcpNativeTls(ref mut connection) => {
-                        let _ = connection.reader.shutdown();
-                        connection.open = false;
-                    }
-                    #[cfg(feature = "tls-rustls")]
-                    ActualConnection::TcpRustls(ref mut connection) => {
-                        let _ = connection.reader.get_mut().shutdown(net::Shutdown::Both);
-                        connection.open = false;
-                    }
-                    #[cfg(unix)]
-                    ActualConnection::Unix(ref mut connection) => {
-                        let _ = connection.sock.shutdown(net::Shutdown::Both);
-                        connection.open = false;
-                    }
-                }
+            ActualConnection::Unix(ref mut connection) => {
+                let _ = connection.sock.shutdown(net::Shutdown::Both);
+                connection.open = false;
             }
         }
-        result
+    }
+
+    /// Fetches a single message from the connection. If the message is a response,
+    /// increment `messages_to_skip` if it wasn't received before a timeout.
+    fn read(&mut self, is_response: bool) -> RedisResult<Value> {
+        loop {
+            let result = match self.con {
+                ActualConnection::Tcp(TcpConnection { ref mut reader, .. }) => {
+                    self.parser.parse_value(reader)
+                }
+                #[cfg(all(feature = "tls-native-tls", not(feature = "tls-rustls")))]
+                ActualConnection::TcpNativeTls(ref mut boxed_tls_connection) => {
+                    let reader = &mut boxed_tls_connection.reader;
+                    self.parser.parse_value(reader)
+                }
+                #[cfg(feature = "tls-rustls")]
+                ActualConnection::TcpRustls(ref mut boxed_tls_connection) => {
+                    let reader = &mut boxed_tls_connection.reader;
+                    self.parser.parse_value(reader)
+                }
+                #[cfg(unix)]
+                ActualConnection::Unix(UnixConnection { ref mut sock, .. }) => {
+                    self.parser.parse_value(sock)
+                }
+            };
+            self.try_send(&result);
+
+            let Err(err) = &result else {
+                if self.messages_to_skip > 0 {
+                    self.messages_to_skip -= 1;
+                    continue;
+                }
+                return result;
+            };
+            let Some(io_error) = err.as_io_error() else {
+                if self.messages_to_skip > 0 {
+                    self.messages_to_skip -= 1;
+                    continue;
+                }
+                return result;
+            };
+            // shutdown connection on protocol error
+            if io_error.kind() == io::ErrorKind::UnexpectedEof {
+                self.close_connection();
+            } else if is_response {
+                self.messages_to_skip += 1;
+            }
+
+            return result;
+        }
     }
 
     /// Sets sender channel for push values.
@@ -1330,7 +1476,7 @@ impl ConnectionLike for Connection {
             return Ok(Value::Nil);
         }
         loop {
-            match self.read_response()? {
+            match self.read(true)? {
                 Value::Push {
                     kind: _kind,
                     data: _data,
@@ -1346,7 +1492,7 @@ impl ConnectionLike for Connection {
 
         self.send_bytes(cmd)?;
         loop {
-            match self.read_response()? {
+            match self.read(true)? {
                 Value::Push {
                     kind: _kind,
                     data: _data,
@@ -1375,7 +1521,7 @@ impl ConnectionLike for Connection {
             // We need to keep processing the rest of the responses in that case,
             // so bailing early with `?` would not be correct.
             // See: https://github.com/redis-rs/redis-rs/issues/436
-            let response = self.read_response();
+            let response = self.read(true);
             match response {
                 Ok(Value::ServerError(err)) => {
                     if idx < offset {
@@ -1538,7 +1684,7 @@ impl<'a> PubSub<'a> {
             return Ok(msg);
         }
         loop {
-            if let Some(msg) = Msg::from_owned_value(self.con.recv_response()?) {
+            if let Some(msg) = Msg::from_owned_value(self.con.read(false)?) {
                 return Ok(msg);
             } else {
                 continue;
