@@ -14,8 +14,10 @@ use std::{
 #[cfg(feature = "aio")]
 use futures::Future;
 #[cfg(feature = "aio")]
-use redis::{aio, cmd, RedisResult};
-use redis::{ConnectionAddr, InfoDict, Pipeline, ProtocolVersion, RedisConnectionInfo, Value};
+use redis::{aio, cmd};
+use redis::{
+    ConnectionAddr, InfoDict, Pipeline, ProtocolVersion, RedisConnectionInfo, RedisResult, Value,
+};
 
 #[cfg(feature = "tls-rustls")]
 use redis::{ClientTlsConfig, TlsCertificates};
@@ -43,7 +45,14 @@ pub fn current_thread_runtime() -> tokio::runtime::Runtime {
 }
 
 #[cfg(feature = "aio")]
-pub fn block_on_all<F, V>(f: F) -> F::Output
+pub enum RuntimeType {
+    Tokio,
+    #[cfg(feature = "async-std-comp")]
+    AsyncStd,
+}
+
+#[cfg(feature = "aio")]
+pub fn block_on_all<F, V>(f: F, runtime: RuntimeType) -> F::Output
 where
     F: Future<Output = RedisResult<V>>,
 {
@@ -72,9 +81,15 @@ where
     let f = futures_util::FutureExt::fuse(f);
     futures::pin_mut!(f, check_future);
 
-    let res = current_thread_runtime().block_on(async {
-        futures::select! {res = f => res, err = check_future => err}
-    });
+    let res = match runtime {
+        RuntimeType::Tokio => current_thread_runtime().block_on(async {
+            futures::select! {res = f => res, err = check_future => err}
+        }),
+        #[cfg(feature = "async-std-comp")]
+        RuntimeType::AsyncStd => block_on_all_using_async_std(async move {
+            futures::select! {res = f => res, err = check_future => err}
+        }),
+    };
 
     let _ = panic::take_hook();
     if CHECK.swap(false, Ordering::Relaxed) {
@@ -85,23 +100,26 @@ where
 }
 
 #[cfg(feature = "aio")]
-#[test]
-fn test_block_on_all_panics_from_spawns() {
-    let result = std::panic::catch_unwind(|| {
-        block_on_all(async {
+#[rstest::rstest]
+#[case::tokio(RuntimeType::Tokio)]
+#[cfg_attr(feature = "async-std-comp", case::async_std(RuntimeType::AsyncStd))]
+#[should_panic]
+fn test_block_on_all_panics_from_spawns(#[case] runtime: RuntimeType) {
+    let _ = block_on_all(
+        async {
             tokio::task::spawn(async {
                 futures_time::task::sleep(futures_time::time::Duration::from_millis(1)).await;
                 panic!("As it should");
             });
             futures_time::task::sleep(futures_time::time::Duration::from_millis(10)).await;
             Ok(())
-        })
-    });
-    assert!(result.is_err());
+        },
+        runtime,
+    );
 }
 
 #[cfg(feature = "async-std-comp")]
-pub fn block_on_all_using_async_std<F>(f: F) -> F::Output
+fn block_on_all_using_async_std<F>(f: F) -> F::Output
 where
     F: Future,
 {
@@ -177,8 +195,8 @@ impl RedisServer {
         RedisServer::with_modules(&[], true)
     }
 
-    pub fn log_file_contents(&self) -> String {
-        std::fs::read_to_string(self.log_file.clone()).unwrap()
+    pub fn log_file_contents(&self) -> Option<String> {
+        std::fs::read_to_string(self.log_file.clone()).ok()
     }
 
     pub fn get_addr(port: u16) -> ConnectionAddr {
@@ -385,6 +403,15 @@ impl RedisServer {
     }
 }
 
+pub fn get_listener_on_free_port() -> TcpListener {
+    let addr = &"127.0.0.1:0".parse::<SocketAddr>().unwrap().into();
+    let socket = Socket::new(Domain::IPV4, Type::STREAM, None).unwrap();
+    socket.set_reuse_address(true).unwrap();
+    socket.bind(addr).unwrap();
+    socket.listen(1).unwrap();
+    TcpListener::from(socket)
+}
+
 /// Finds a random open port available for listening at, by spawning a TCP server with
 /// port "zero" (which prompts the OS to just use any available port). Between calling
 /// this function and trying to bind to this port, the port may be given to another
@@ -392,12 +419,7 @@ impl RedisServer {
 /// mostly okay).
 pub fn get_random_available_port() -> u16 {
     for _ in 0..10000 {
-        let addr = &"127.0.0.1:0".parse::<SocketAddr>().unwrap().into();
-        let socket = Socket::new(Domain::IPV4, Type::STREAM, None).unwrap();
-        socket.set_reuse_address(true).unwrap();
-        socket.bind(addr).unwrap();
-        socket.listen(1).unwrap();
-        let listener = TcpListener::from(socket);
+        let listener = get_listener_on_free_port();
         let port = listener.local_addr().unwrap().port();
         if port < 55535 {
             return port;
@@ -460,11 +482,8 @@ impl TestContext {
             },
         );
 
-        #[cfg(feature = "tls-rustls")]
         let client =
             build_single_client(server.connection_info(), &server.tls_paths, mtls_enabled).unwrap();
-        #[cfg(not(feature = "tls-rustls"))]
-        let client = redis::Client::open(server.connection_info()).unwrap();
 
         let mut con;
 
@@ -478,13 +497,13 @@ impl TestContext {
                         retries += 1;
                         if retries > 100000 {
                             panic!(
-                                "Tried to connect too many times, last error: {err}, logfile: {}",
+                                "Tried to connect too many times, last error: {err}, logfile: {:?}",
                                 server.log_file_contents()
                             );
                         }
                     } else {
                         panic!(
-                            "Could not connect: {err}, logfile: {}",
+                            "Could not connect: {err}, logfile: {:?}",
                             server.log_file_contents()
                         );
                     }
@@ -508,6 +527,12 @@ impl TestContext {
         self.client.get_connection().unwrap()
     }
 
+    #[cfg(feature = "aio")]
+    #[allow(deprecated)]
+    pub async fn deprecated_async_connection(&self) -> RedisResult<redis::aio::Connection> {
+        self.client.get_async_connection().await
+    }
+
     #[cfg(feature = "aio")]
     pub async fn async_connection(&self) -> RedisResult<redis::aio::MultiplexedConnection> {
         self.client.get_multiplexed_async_connection().await
@@ -518,24 +543,10 @@ impl TestContext {
         self.client.get_async_pubsub().await
     }
 
-    #[cfg(feature = "async-std-comp")]
-    pub async fn async_connection_async_std(
-        &self,
-    ) -> RedisResult<redis::aio::MultiplexedConnection> {
-        self.client.get_multiplexed_async_std_connection().await
-    }
-
     pub fn stop_server(&mut self) {
         self.server.stop();
     }
 
-    #[cfg(feature = "tokio-comp")]
-    pub async fn multiplexed_async_connection(
-        &self,
-    ) -> RedisResult<redis::aio::MultiplexedConnection> {
-        self.multiplexed_async_connection_tokio().await
-    }
-
     #[cfg(feature = "tokio-comp")]
     pub async fn multiplexed_async_connection_tokio(
         &self,
@@ -543,13 +554,6 @@ impl TestContext {
         self.client.get_multiplexed_tokio_connection().await
     }
 
-    #[cfg(feature = "async-std-comp")]
-    pub async fn multiplexed_async_connection_async_std(
-        &self,
-    ) -> RedisResult<redis::aio::MultiplexedConnection> {
-        self.client.get_multiplexed_async_std_connection().await
-    }
-
     pub fn get_version(&self) -> Version {
         let mut conn = self.connection();
         get_version(&mut conn)
@@ -655,7 +659,7 @@ pub fn build_keys_and_certs_for_tls(tempdir: &TempDir) -> TlsFilePaths {
             .arg("genrsa")
             .arg("-out")
             .arg(name)
-            .arg(&format!("{size}"))
+            .arg(format!("{size}"))
             .stdout(process::Stdio::piped())
             .stderr(process::Stdio::piped())
             .spawn()
@@ -822,6 +826,15 @@ pub(crate) fn build_single_client<T: redis::IntoConnectionInfo>(
     }
 }
 
+#[cfg(not(feature = "tls-rustls"))]
+pub(crate) fn build_single_client<T: redis::IntoConnectionInfo>(
+    connection_info: T,
+    _tls_file_params: &Option<TlsFilePaths>,
+    _mtls_enabled: bool,
+) -> RedisResult<redis::Client> {
+    redis::Client::open(connection_info)
+}
+
 #[cfg(feature = "tls-rustls")]
 pub(crate) mod mtls_test {
     use super::*;
@@ -909,3 +922,16 @@ pub async fn kill_client_async(
 
     Ok(())
 }
+
+pub async fn spawn<T>(fut: impl std::future::Future<Output = T> + Send + Sync + 'static) -> T
+where
+    T: Send + 'static,
+{
+    match tokio::runtime::Handle::try_current() {
+        Ok(tokio_runtime) => tokio_runtime.spawn(fut).await.unwrap(),
+        #[cfg(feature = "async-std-comp")]
+        Err(_) => async_std::task::spawn(fut).await,
+        #[cfg(not(feature = "async-std-comp"))]
+        Err(_) => unreachable!(),
+    }
+}
