@@ -13,7 +13,8 @@ mod basic {
     };
     use std::collections::{BTreeMap, BTreeSet};
     use std::collections::{HashMap, HashSet};
-    use std::thread::{sleep, spawn};
+    use std::io::Read;
+    use std::thread::{self, sleep, spawn};
     use std::time::{Duration, SystemTime, UNIX_EPOCH};
     use std::vec;
 
@@ -534,6 +535,7 @@ mod basic {
             let _: () = con.append(format!("test/{i}"), i).unwrap();
             let _: () = con.append(format!("other/{i}"), i).unwrap();
         }
+        let _: () = con.hset("test-hset", "test-field", "test-value").unwrap();
 
         // scan with pattern
         let opts = ScanOptions::default().with_count(20).with_pattern("test/*");
@@ -544,7 +546,17 @@ mod basic {
         let opts = ScanOptions::default();
         let values = con.scan_options::<String>(opts).unwrap();
         let values: Vec<_> = values.collect();
+        assert_eq!(values.len(), 41);
+        // scan with type string
+        let opts = ScanOptions::default().with_type("string");
+        let values = con.scan_options::<String>(opts).unwrap();
+        let values: Vec<_> = values.collect();
         assert_eq!(values.len(), 40);
+        // scan with type hash
+        let opts = ScanOptions::default().with_type("hash");
+        let values = con.scan_options::<String>(opts).unwrap();
+        let values: Vec<_> = values.collect();
+        assert_eq!(values.len(), 1);
     }
 
     #[test]
@@ -880,6 +892,46 @@ mod basic {
         }
     }
 
+    #[test]
+    fn test_pubsub_handles_timeout() {
+        let ctx = TestContext::new();
+        let mut con = ctx.connection();
+
+        // Connection for subscriber api
+        let mut pubsub_con = ctx.connection();
+        pubsub_con
+            .set_read_timeout(Some(Duration::from_millis(5)))
+            .unwrap();
+        let mut pubsub_con = pubsub_con.as_pubsub();
+
+        pubsub_con.subscribe("foo").unwrap();
+        let err = pubsub_con.get_message().unwrap_err();
+        assert!(err.is_timeout());
+
+        let _: () = con.publish("foo", "bar").unwrap();
+
+        let msg = pubsub_con.get_message().unwrap();
+        assert_eq!(msg.get_channel(), Ok("foo".to_string()));
+        assert_eq!(msg.get_payload(), Ok("bar".to_string()));
+    }
+
+    #[test]
+    fn pub_sub_subscription_to_multiple_channels() {
+        let ctx = TestContext::new();
+        let mut conn = ctx.connection();
+        let mut pubsub_conn = conn.as_pubsub();
+        pubsub_conn.subscribe(&["phonewave", "foo", "bar"]).unwrap();
+        let mut publish_conn = ctx.connection();
+
+        let _: () = publish_conn.publish("phonewave", "banana").unwrap();
+        let msg_payload: String = pubsub_conn.get_message().unwrap().get_payload().unwrap();
+        assert_eq!("banana".to_string(), msg_payload);
+
+        let _: () = publish_conn.publish("foo", "foobar").unwrap();
+        let msg_payload: String = pubsub_conn.get_message().unwrap().get_payload().unwrap();
+        assert_eq!("foobar".to_string(), msg_payload);
+    }
+
     #[test]
     fn test_pubsub_unsubscribe() {
         let ctx = TestContext::new();
@@ -1672,6 +1724,105 @@ mod basic {
         assert_eq!(expire_time_milliseconds, now * 1000 + 12_000);
     }
 
+    #[test]
+    fn test_timeout_leaves_usable_connection() {
+        let ctx = TestContext::new();
+        let mut con = ctx.connection();
+
+        () = con.set("key", "value").unwrap();
+
+        con.set_read_timeout(Some(Duration::from_millis(1)))
+            .unwrap();
+
+        // send multiple requests that timeout.
+        for _ in 0..3 {
+            let res = con.blpop::<_, Value>("foo", 0.03);
+            assert!(res.unwrap_err().is_timeout());
+            assert!(con.is_open());
+        }
+
+        sleep(Duration::from_millis(100));
+
+        // we might need to flush temporary errors from the system
+        for _ in 0..100 {
+            if cmd("PING").exec(&mut con).is_ok() {
+                break;
+            }
+            sleep(Duration::from_millis(1));
+        }
+
+        let res: String = con.get("key").unwrap();
+        assert_eq!(res, "value");
+    }
+
+    #[test]
+    fn test_timeout_in_middle_of_message_leaves_connection_usable() {
+        use std::io::Write;
+
+        fn fake_redis(listener: std::net::TcpListener) {
+            let mut stream = listener.incoming().next().unwrap().unwrap();
+
+            let mut reader = std::io::BufReader::new(stream.try_clone().unwrap());
+
+            // handle initial handshake if sent
+            #[cfg(not(feature = "disable-client-setinfo"))]
+            {
+                let mut pipeline = redis::pipe();
+                pipeline
+                    .cmd("CLIENT")
+                    .arg("SETINFO")
+                    .arg("LIB-NAME")
+                    .arg("redis-rs");
+                pipeline
+                    .cmd("CLIENT")
+                    .arg("SETINFO")
+                    .arg("LIB-VER")
+                    .arg(env!("CARGO_PKG_VERSION"));
+                let expected_length = pipeline.get_packed_pipeline().len();
+                let mut buf = vec![0; expected_length];
+                reader.read_exact(&mut buf).unwrap();
+                stream.write_all(b"$2\r\nOK\r\n$2\r\nOK\r\n").unwrap();
+            }
+
+            // reply with canned responses to known requests.
+            loop {
+                let expected_length = cmd("GET").arg("key1").get_packed_command().len();
+                let mut buf = vec![0; expected_length];
+                reader.read_exact(&mut buf).unwrap();
+
+                if buf == cmd("GET").arg("key1").get_packed_command() {
+                    // split the response so half is sent before the timeout
+                    stream.write_all(b"$4\r\nk").unwrap();
+                    sleep(Duration::from_millis(100));
+                    stream.write_all(b"ey1\r\n").unwrap();
+                } else if buf == cmd("GET").arg("key2").get_packed_command() {
+                    stream.write_all(b"$4\r\nkey2\r\n").unwrap();
+                    return;
+                } else {
+                    panic!("Invalid command {}", String::from_utf8(buf).unwrap());
+                }
+            }
+        }
+
+        let listener = get_listener_on_free_port();
+        let port = listener.local_addr().unwrap().port();
+        thread::spawn(move || fake_redis(listener));
+
+        let client = redis::Client::open(format!("redis://127.0.0.1:{port}/")).unwrap();
+        let mut con = client.get_connection().unwrap();
+        con.set_read_timeout(Some(Duration::from_millis(10)))
+            .unwrap();
+
+        let res = con.get::<_, Value>("key1");
+        assert!(res.unwrap_err().is_timeout());
+        assert!(con.is_open());
+
+        sleep(Duration::from_millis(100));
+
+        let value = con.get::<_, String>("key2").unwrap();
+        assert_eq!(value, "key2");
+    }
+
     #[test]
     fn test_blocking_sorted_set_api() {
         let ctx = TestContext::new();
@@ -1873,4 +2024,32 @@ mod basic {
             (kind, data)
         );
     }
+
+    #[test]
+    fn test_select_db() {
+        let ctx = TestContext::new();
+        let mut connection_info = ctx.client.get_connection_info().clone();
+        connection_info.redis.db = 5;
+        let client = redis::Client::open(connection_info).unwrap();
+        let mut connection = client.get_connection().unwrap();
+        let info: String = redis::cmd("CLIENT")
+            .arg("info")
+            .query(&mut connection)
+            .unwrap();
+        assert!(info.contains("db=5"));
+    }
+
+    #[test]
+    fn test_client_getname() {
+        let ctx = TestContext::new();
+        let mut con = ctx.connection();
+
+        redis::cmd("CLIENT")
+            .arg("SETNAME")
+            .arg("connection-name")
+            .exec(&mut con)
+            .unwrap();
+        let res: String = con.client_getname().unwrap();
+        assert_eq!(res, "connection-name");
+    }
 }
